@@ -31,10 +31,11 @@ for my $list (qw(user_ids subkeys user_attributes)) {
         },
     );
 
-    __PACKAGE__->meta->add_method($list => sub {
+    no strict 'refs';
+    *{$list} = sub {
         my $self = shift;
         return wantarray ? @{$self->$ref(@_)} : $self->$ref(@_);
-    });
+    };
 }
 
 has $_ => (
@@ -47,8 +48,6 @@ has $_ => (
 sub compare {
   my ($self, $other, $deep) = @_;
 
-  # not comparing local_id because it is meaningless in modern
-  # versions of GnuPG.
   my @comparison_fields = qw (
      owner_trust
   );
