@@ -15,7 +15,8 @@
 #
 
 package GnuPG::Revoker;
-use Any::Moose;
+use Moo;
+use MooX::late;
 
 has [qw(
          algo_num
@@ -70,8 +71,6 @@ sub compare {
   return 1;
 }
 
-__PACKAGE__->meta->make_immutable;
-
 1;
 
 __END__
